/***************************************************************************//**
 *   @file   ADF4118.c
 *   @brief  Implementation of ADF4118 Driver.
 *   @author DBogdan (dragos.bogdan@analog.com)
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
*   SVN Revision: 767
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "ADF4118.h"		// ADF4118 definitions.
#include "ADF4118_cfg.h"    // ADF4118_cfg definitions.
#include "Communication.h"	// Communication definitions.

/******************************************************************************/
/************************ Variables Definitions *******************************/
/******************************************************************************/
unsigned long regR    = 0;
unsigned long regN    = 0;
unsigned long regFunc = 0;
unsigned long regInit = 0;

/***************************************************************************//**
 * @brief Initializes the device.
 *
 * @return status - Result of the initialization procedure.
 *					Example: 0x0 - SPI peripheral was not initialized.
 *				  			 0x1 - SPI peripheral is initialized.
*******************************************************************************/
unsigned char ADF4118_Init(void)
{
	unsigned char status = 0x0;
    
	status = SPI_Init(0, 1000000, 0, 1);
	/* Initialize ADF4118 registers. */
    regInit = ADF4118_FUNC_PWR_DWN(ADF4118_st.powerDown) | 
              ADF4118_FUNC_TMR_CNT_CTRL(ADF4118_st.timerCounterCtrl) | 
              ADF4118_FUNC_FAST_MODE(ADF4118_st.fastLock) | 
              ADF4118_FUNC_CHARGE_PUMP(ADF4118_st.cpThreeState) | 
              ADF4118_FUNC_PH_DETECT_POL(ADF4118_st.phaseDetectorPolarity) | 
              ADF4118_FUNC_MUXOUT(ADF4118_st.muxControl);
    ADF4118_Write(ADF4118_REG_INITIALIZATION_LATCH | regInit);
    regR = ADF4118_R_CNT_LD_PRECISION(ADF4118_st.lockDetectPrecision);                  
    ADF4118_Write(ADF4118_REG_R_COUNTER | regR);
    regN = ADF4118_N_CNT_CP_GAIN(ADF4118_st.cpGain);
    ADF4118_Write(ADF4118_REG_N_COUNTER | regN);
    regFunc = regInit;
    
	return(status);
}

/***************************************************************************//**
 * @brief Writes data into a register.
 *
 * @param data - Data value to write.
 *
 * @return None.
*******************************************************************************/
void ADF4118_Write(unsigned long data)
{
	unsigned char* dataPointer  = (unsigned char*)&data;
	unsigned char spiData[3]    = {0, 0, 0};
    unsigned char slaveDeviceId = 1;
    
	spiData[0] = dataPointer[2];
   	spiData[1] = dataPointer[1];
   	spiData[2] = dataPointer[0];
	SPI_Write(slaveDeviceId, spiData, 3);
}

/***************************************************************************//**
 * @brief Powers down or powers up the device.
 *
 * @param powerMode - Power option.
 *                    Example: 0 - powers down the device;
 *                             1 - power up the device.
 *
 * @return None.
*******************************************************************************/
void ADF4118_Power(unsigned char powerMode)
{
    if(powerMode)
    {
        ADF4118_Write(ADF4118_REG_FUNCTION_LATCH |
                      regFunc |
                      ADF4118_FUNC_PWR_DWN(ADF4118_PWR_NORMAL_OPERATION));
    }
    else
    {
        ADF4118_Write(ADF4118_REG_FUNCTION_LATCH |
                      regFunc |
                      ADF4118_FUNC_PWR_DWN(ADF4118_PWR_SYNCH_POWER_DOWN));
    }
}

/***************************************************************************//**
 * @brief Increases the R counter value until the ADF4118_MAX_FREQ_PFD is
 *        greater than PFD frequency.
 *
 * @param rCounter - R counter value.
 *
 * @return rCounter - modified R counter value.
*******************************************************************************/
unsigned short ADF4118_TuneRcounter(unsigned short rCounter)
{
	unsigned long frequencyPfd = 0;	// PFD frequency
	
	do
	{
		rCounter++;
		frequencyPfd = ADF4118_st.refIn / rCounter;
	}
	while(frequencyPfd > ADF4118_MAX_FREQ_PFD);
    
    return rCounter;
}

/***************************************************************************//**
 * @brief Sets the ADF4118 frequency.
 *
 * @param frequency - The desired frequency value.
 *
 * @return calculatedFrequency - The actual frequency value that was set.
*******************************************************************************/
unsigned long long ADF4118_SetFrequency(unsigned long long frequency)
{
    unsigned long long vcoFrequency        = 0; // VCO frequency
    unsigned long      frequencyPfd        = 0;	// PFD frequency
    unsigned long      freqRatio           = 0; // VCOfreq / PFDfreq
    unsigned long long calculatedFrequency = 0; // Actual VCO frequency
    unsigned short	   rCounterValue 	   = 0; // Value for R counter
    unsigned short     a                   = 0; // Value for A counter
    unsigned short     b                   = 0; // Value for B counter
    
    /* Force "frequency" parameter to the [100MHz...3GHz] interval. */
    if(frequency <= ADF4118_MAX_FREQ_VCO)
    {
        if(frequency >= ADF4118_MIN_FREQ_VCO)
        {
            vcoFrequency = frequency;
        }
        else
        {
            vcoFrequency = ADF4118_MIN_FREQ_VCO;
        }
    }
    else
    {
        vcoFrequency = ADF4118_MAX_FREQ_VCO;
    }    
    do
    {
        /* Get the actual PFD frequency. */
        rCounterValue = ADF4118_TuneRcounter(rCounterValue);
        frequencyPfd = ADF4118_st.refIn / rCounterValue;
        /* Find the values for Counter A and Counter B using VCO frequency and 
        PFD frequency. */
        freqRatio = (unsigned short)((float)vcoFrequency / frequencyPfd + 0.5);
        b = freqRatio / ADF4118_DUAL_MODULUS_P;
        a = freqRatio % ADF4118_DUAL_MODULUS_P;
    }while(a > b); // B must be greater or equal to A
    /* Find the actual VCO frequency. */
    calculatedFrequency = ((b * ADF4118_DUAL_MODULUS_P) + a) * frequencyPfd;
    /* Load the saved values into ADF4118 registers using Counter Reset
    Method. */
    ADF4118_Write(ADF4118_REG_FUNCTION_LATCH | 
                  regFunc | 
                  ADF4118_FUNC_COUNTER_RESET(ADF4118_COUNTER_RST_ENABLE));              
    ADF4118_Write(ADF4118_REG_R_COUNTER | 
                  regR |
                  ADF4118_R_CNT_14_BIT_REF_COUNTER(rCounterValue));
    ADF4118_Write(ADF4118_REG_N_COUNTER | 
                  regN | 
                  ADF4118_N_CNT_13_BIT_B_COUNTER(b) | 
                  ADF4118_N_CNT_5_BIT_A_COUNTER(a));
    ADF4118_Write(ADF4118_REG_FUNCTION_LATCH | 
                  regFunc | 
                  ADF4118_FUNC_COUNTER_RESET(ADF4118_COUNTER_RST_DISABLE));

    return calculatedFrequency;
}
