/***************************************************************************//**
 *   @file   ADF4118.h
 *   @brief  Header file of ADF4118 Driver.
 *   @author DBogdan (dragos.bogdan@analog.com)
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
*   SVN Revision: 767
*******************************************************************************/
#ifndef __ADF4118_H__
#define __ADF4118_H__

/******************************************************************************/
/******************************** ADF4118 *************************************/
/******************************************************************************/

/* ADF4118 latch control bits  */
#define ADF4118_REG_R_COUNTER					0
#define ADF4118_REG_N_COUNTER					1
#define ADF4118_REG_FUNCTION_LATCH				2
#define ADF4118_REG_INITIALIZATION_LATCH		3

/* ADF4118_REG_R_COUNTER definition */
#define ADF4118_R_CNT_LD_PRECISION(x)	        ((x) << 20)
#define ADF4118_R_CNT_14_BIT_REF_COUNTER(x)     ((0x3FFF & (x)) << 2)

/* ADF4118_R_CNT_LD_PRECISION options */
#define ADF4118_LOCK_DETECT_3_CYCLES        0
#define ADF4118_LOCK_DETECT_5_CYCLES        1

/* ADF4118_REG_N_COUNTER definition */
#define ADF4118_N_CNT_CP_GAIN(x)                ((x) << 20)
#define ADF4118_N_CNT_13_BIT_B_COUNTER(x)       ((0x1FFF & (x)) << 7)
#define ADF4118_N_CNT_5_BIT_A_COUNTER(x)        ((0x1F & (x)) << 2)

/* ADF4118_N_CNT_CP_GAIN options */
#define ADF4118_CP_GAIN_250_MICRO_A          0
#define ADF4118_CP_GAIN_1_MILLI_A            1

/* ADF4118_REG_FUNCTION_LATCH / ADF4118_REG_INITIALIZATION_LATCH definition */
#define ADF4118_FUNC_PWR_DWN(x)         (((x) & 1) << 3)|(((x) & 2) << 19)
#define ADF4118_FUNC_TMR_CNT_CTRL(x)    ((0xF & (x)) << 11)
#define ADF4118_FUNC_FAST_MODE(x)       (((x) & 0x1) << 9) | (((x) & 0x2) << 11)
#define ADF4118_FUNC_CHARGE_PUMP(x)     ((x) << 8)
#define ADF4118_FUNC_PH_DETECT_POL(x)	((x) << 7)
#define ADF4118_FUNC_MUXOUT(x)          (((x) & 0x7) << 4)
#define ADF4118_FUNC_COUNTER_RESET(x)   ((x) << 2)

/* ADF4118_FUNC_PWR_DWN(x) options */
#define ADF4118_PWR_NORMAL_OPERATION			0
#define ADF4118_PWR_ASYNCH_POWER_DOWN		    1
#define ADF4118_PWR_SYNCH_POWER_DOWN		    2

/* ADF4118_FUNC_FAST_MODE(x) options */
#define ADF4118_FASTLOCK_DISABLED			    0
#define ADF4118_FASTLOCK_MODE_1				    1
#define ADF4118_FASTLOCK_MODE_2				    3

/* ADF4118_FUNC_CHARGE_PUMP(x) options */
#define ADF4118_CP_NORMAL_OUTPUT                0
#define ADF4118_CP_THREE_STATE                  1

/* ADF4118_FUNC_PH_DETECT_POL(x) options */
#define ADF4118_PH_DETECT_POL_NEGATIVE          0
#define ADF4118_PH_DETECT_POL_POSITIVE          1

/* ADF4118_FUNC_MUXOUT(x) options */
#define ADF4118_MUX_THREE_STATE_OUTPUT			0
#define ADF4118_MUX_DIGITAL_LOCK_DETECT			1
#define ADF4118_MUX_N_DIVIDER_OUTPUT			2
#define ADF4118_MUX_AVDD						3
#define ADF4118_MUX_R_DIVIDER_OUTPUT			4
#define ADF4118_MUX_ANALOG_LOCK_DETECT			5
#define ADF4118_MUX_SERIAL_DATA_OUTPUT			6
#define ADF4118_MUX_DGND						7

/* ADF4118_FUNC_COUNTER_RESET(x) options */
#define ADF4118_COUNTER_RST_DISABLE             0
#define ADF4118_COUNTER_RST_ENABLE              1

/* ADF4118 Specifications */
#define ADF4118_DUAL_MODULUS_P              32
#define ADF4118_MAX_FREQ_PFD                55000000     // Hz
#define ADF4118_MAX_FREQ_VCO                3000000000   // Hz
#define ADF4118_MIN_FREQ_VCO                100000000    // Hz
#define ADF4118_MAX_FREQ_REF_IN             100000000    // Hz

/*****************************************************************************/
/************************** Types Declarations *******************************/
/*****************************************************************************/
struct ADF4118_settings
{
    /* External Configuration */
    unsigned long refIn; // Reference frequency input
    
    /* Initialization Latch */
    unsigned char powerDown;
    unsigned char timerCounterCtrl;
    unsigned char fastLock;    
    unsigned char cpThreeState;
    unsigned char phaseDetectorPolarity;
    unsigned char muxControl;
    
    /* N Counter Latch */
    unsigned char cpGain;
    
    /* R Counter Latch */
    unsigned char lockDetectPrecision;
};

/******************************************************************************/
/************************ Functions Declarations ******************************/
/******************************************************************************/

/*! Initializes the device. */
unsigned char ADF4118_Init(void);

/*! Writes data into a register. */
void ADF4118_Write(unsigned long data);

/*! Powers down or powers up the device. */
void ADF4118_Power(unsigned char powerMode);

/*! Sets the ADF4118 frequency. */
unsigned long long ADF4118_SetFrequency(unsigned long long frequency);

#endif // __ADF4118_H__
